--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: ARepKey                                             **
-- **    Module: ARepKey (root)                                            **
-- **    Target device: CoolRunner II (XC2C256/TQ144) [DIGILENT]           **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity ARepKey is port ( Clk : in std_logic ;
                         Reset : in std_logic ;
                         KeyIn : in std_logic ;
                         Seg_A : out std_logic ;
                         Seg_B : out std_logic ;
                         Seg_C : out std_logic ;
                         Seg_D : out std_logic ;
                         Seg_E : out std_logic ;
                         Seg_F : out std_logic ;
                         Seg_G : out std_logic ;
                         Seg_K : out std_logic ;
                         DSel0 : out std_logic ;
                         DSel1 : out std_logic ;
                         DSel2 : out std_logic ;
                         DSel3 : out std_logic ) ;
end ARepKey ;

architecture Behavioral of ARepKey is

  constant FDivCntSizeParam : integer := 8 ;
  constant DisplayNegParam : std_logic := '0' ;
  constant DisplayWAParam : std_logic := '0' ;

  component FDivider is generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component BCD4DigCnt port ( Clk : in std_logic ;
                              Reset : in std_logic ;
                              Enable : in std_logic ;
                              Cnt1 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt10 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt100 : out std_logic_vector ( 3 downto 0 ) ;
                              Cnt1000 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  type AutomatStateType is ( IdleState ,
                             StartTimeState ,
                             WaitTimeState ,
                             KeyReady1State ,
                             KeyReady2State ,
                             StartWaitTimeFreeKeyState ,
                             WaitTimeFreeKeyState ,
                             StartAutoRepKeyState ,
                             AutoRepKeyState ) ;

  constant StandardDelay : std_logic_vector ( 4 downto 0 ) := "00011" ;
  constant StartRepeatDelay : std_logic_vector ( 4 downto 0 ) := "11100" ;
  constant RepeatDelay : std_logic_vector ( 4 downto 0 ) := "00100" ;

  signal AutomatState : AutomatStateType ;
  signal Cnt1 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt10 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt100 : std_logic_vector ( 3 downto 0 ) ;
  signal Cnt1000 : std_logic_vector ( 3 downto 0 ) ;
  signal DClk : std_logic ;
  signal TClk : std_logic ;
  signal FClk : std_logic ;
  signal FDiv : std_logic_vector ( 5 downto 0 ) ;
  signal DelayCnt : std_logic_vector ( 4 downto 0 ) ;
  signal DelayCntEnable : std_logic ;
  signal DelayCntClear : std_logic ;
  signal CntClk : std_logic ;

begin
  FDivInstance : FDivider generic map ( CntSize => FDivCntSizeParam )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 2 ) ;
  TClk <= FDiv ( 5 ) ;

  DelatCntInstance : process ( TClk , Reset , DelayCntClear , DelayCntEnable )
  begin
    if Reset = '0' or DelayCntClear = '0' then
      DelayCnt <= ( others => '0' ) ;
    else
      if DelayCntEnable = '1' then
        if TClk'event and TClk = '1' then
          DelayCnt <= DelayCnt + 1 ;
        end if ;
      end if ;
    end if ;
  end process ;

  AutomInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      AutomatState <= IdleState ;
    else
      if Clk'event and Clk = '1' then
        case AutomatState is
          when IdleState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '1' ;
            CntClk <= '0' ;
            if KeyIn = '1' then
              AutomatState <= IdleState ;
            else
              AutomatState <= StartTimeState ;
            end if ;
          when StartTimeState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '0' ;
            CntClk <= '0' ;
            AutomatState <= WaitTimeState ;
          when WaitTimeState =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            CntClk <= '0' ;
            if KeyIn = '1' then
              AutomatState <= IdleState ;
            else
              if DelayCnt = StandardDelay then
                AutomatState <= KeyReady1State ;
              else
                AutomatState <= WaitTimeState ;
              end if ;
            end if ;
          when KeyReady1State =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            CntClk <= '1' ;
            AutomatState <= KeyReady2State ;
          when KeyReady2State =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            CntClk <= '0' ;
            if KeyIn = '1' then
              AutomatState <= StartWaitTimeFreeKeyState ;
            else
              if DelayCnt = StartRepeatDelay then
                AutomatState <= StartAutoRepKeyState ;
              else
                AutomatState <= KeyReady2State ;
              end if ;
            end if ;
          when StartWaitTimeFreeKeyState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '0' ;
            CntClk <= '0' ;
            AutomatState <= WaitTimeFreeKeyState ;
          when WaitTimeFreeKeyState =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            CntClk <= '0' ;
            if DelayCnt = StandardDelay then
              AutomatState <= IdleState ;
            else
              AutomatState <= WaitTimeFreeKeyState ;
            end if ;
          when StartAutoRepKeyState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '0' ;
            CntClk <= '1' ;
            AutomatState <= AutoRepKeyState ;
          when AutoRepKeyState =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            CntClk <= '0' ;
            if KeyIn = '1' then
              AutomatState <= StartWaitTimeFreeKeyState ;
            else
              if DelayCnt = RepeatDelay then
                AutomatState <= StartAutoRepKeyState ;
              else
                AutomatState <= AutoRepKeyState ;
              end if ;
            end if ;
        end case ;
      end if ;
    end if ;
  end process ;

  BCDCntInstance : BCD4DigCnt port map ( Clk => CntClk ,
                                         Reset => Reset ,
                                         Enable => '1' ,
                                         Cnt1 => Cnt1 ,
                                         Cnt10 => Cnt10 ,
                                         Cnt100 => Cnt100 ,
                                         Cnt1000 => Cnt1000 ) ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Neg => DisplayNegParam ,
                                            WA => DisplayWAParam ,
                                            Tetr0 => Cnt1 ,
                                            Tetr1 => Cnt10 ,
                                            Tetr2 => Cnt100 ,
                                            Tetr3 => Cnt1000 ,
                                            Dot => "1111" ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

end Behavioral ;
